"""Module for the root command: vulnz."""
from ostorlab.cli.vulnz.vulnz import vulnz
from ostorlab.cli.vulnz import list as list_cli
from ostorlab.cli.vulnz import describe as describe_cli
from ostorlab.cli.vulnz import dump
