# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/scanner/proto/assets/link.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(ostorlab/scanner/proto/assets/link.proto\x12\"ostorlab.scanner.proto.assets.link\"%\n\x06Header\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"%\n\x06\x43ookie\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"7\n\x0cLocalStorage\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"6\n\tFormInput\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\"V\n\x04\x46orm\x12=\n\x06inputs\x18\x01 \x03(\x0b\x32-.ostorlab.scanner.proto.assets.link.FormInput\x12\x0f\n\x07\x65nctype\x18\x02 \x01(\t\">\n\x0e\x46ormCredential\x12\r\n\x05login\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\"\"\n\x10ScriptCredential\x12\x0e\n\x06script\x18\x01 \x01(\t\"\xbf\x04\n\x07Message\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06method\x18\x02 \x01(\t\x12\x0e\n\x04\x62ody\x18\x03 \x01(\x0cH\x00\x12\x38\n\x04\x66orm\x18\x04 \x01(\x0b\x32(.ostorlab.scanner.proto.assets.link.FormH\x00\x12\x41\n\rextra_headers\x18\x05 \x03(\x0b\x32*.ostorlab.scanner.proto.assets.link.Header\x12;\n\x07\x63ookies\x18\x06 \x03(\x0b\x32*.ostorlab.scanner.proto.assets.link.Cookie\x12M\n\x0f\x66orm_credential\x18\x07 \x01(\x0b\x32\x32.ostorlab.scanner.proto.assets.link.FormCredentialH\x01\x12Q\n\x11script_credential\x18\x08 \x01(\x0b\x32\x34.ostorlab.scanner.proto.assets.link.ScriptCredentialH\x01\x12;\n\x06parent\x18\t \x01(\x0b\x32+.ostorlab.scanner.proto.assets.link.Message\x12L\n\x12localstorage_items\x18\n \x03(\x0b\x32\x30.ostorlab.scanner.proto.assets.link.LocalStorageB\x0c\n\nbody_oneofB\x12\n\x10\x63redential_oneof')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.scanner.proto.assets.link_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=80
  _HEADER._serialized_end=117
  _COOKIE._serialized_start=119
  _COOKIE._serialized_end=156
  _LOCALSTORAGE._serialized_start=158
  _LOCALSTORAGE._serialized_end=213
  _FORMINPUT._serialized_start=215
  _FORMINPUT._serialized_end=269
  _FORM._serialized_start=271
  _FORM._serialized_end=357
  _FORMCREDENTIAL._serialized_start=359
  _FORMCREDENTIAL._serialized_end=421
  _SCRIPTCREDENTIAL._serialized_start=423
  _SCRIPTCREDENTIAL._serialized_end=457
  _MESSAGE._serialized_start=460
  _MESSAGE._serialized_end=1035
# @@protoc_insertion_point(module_scope)
